@@ -33,6 +33,16 @@ class LowerDimensionalObject(u.Quantity, BaseNDClass, HeaderMixinClass):
     Generic class for 1D and 2D objects.
     """
 
+    def _new_view(self, obj=None, unit=None, finalize=True, **kwargs):
+        """
+        kwargs are passed to _new_view of other object; only known keyword as of June 2023 is ``propagate_info``
+        """
+        # FORCE finalization to hack around https://github.com/astropy/astropy/issues/14514#issuecomment-1463935711
+        try:
+            return super(LowerDimensionalObject, self)._new_view(obj=obj, unit=unit, finalize=True, **kwargs)
+        except TypeError:
+            return super(LowerDimensionalObject, self)._new_view(obj=obj, unit=unit, **kwargs)
+
     @property
     def hdu(self):
         if self.wcs is None:
@@ -41,7 +51,7 @@ class LowerDimensionalObject(u.Quantity, BaseNDClass, HeaderMixinClass):
             hdu = PrimaryHDU(self.value, header=self.header)
         hdu.header['BUNIT'] = self.unit.to_string(format='fits')
 
-        if 'beam' in self.meta:
+        if self.meta is not None and 'beam' in self.meta:
             hdu.header.update(self.meta['beam'].to_header_keywords())
 
         return hdu
@@ -246,7 +256,7 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
             raise ValueError("value should be a 2-d array")
 
         if wcs is not None and wcs.wcs.naxis != 2:
-            raise ValueError("wcs should have two dimension")
+            raise ValueError("wcs should have two dimensions")
 
         self = u.Quantity.__new__(cls, value, unit=unit, dtype=dtype,
                                   copy=copy).view(cls)
@@ -367,13 +377,20 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
         return newproj
 
     @staticmethod
-    def from_hdu(hdu):
+    def from_hdu(hdu, ext=0):
         '''
         Return a projection from a FITS HDU.
+
+        Parameters
+        -----------
+        ext : int
+            The integer index to load when given an :class:`astropy.io.fits.HDUList`.
+            Default is 0 (the first HDU in the list.
+
         '''
         if isinstance(hdu, HDUList):
             hdul = hdu
-            hdu = hdul[0]
+            hdu = hdul[ext]
 
         if not len(hdu.data.shape) == 2:
             raise ValueError("HDU must contain two-dimensional data.")
@@ -592,7 +609,7 @@ class BaseOneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
             raise ValueError("value should be a 1-d array")
 
         if wcs is not None and wcs.wcs.naxis != 1:
-            raise ValueError("wcs should have two dimension")
+            raise ValueError("wcs should have one dimension")
 
         self = u.Quantity.__new__(cls, value, unit=unit, dtype=dtype,
                                   copy=copy).view(cls)
@@ -625,14 +642,21 @@ class BaseOneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
         return '{0}{1}{2:s}>'.format(prefixstr, arrstr, self._unitstr)
 
     @staticmethod
-    def from_hdu(hdu):
+    def from_hdu(hdu, ext=0):
         '''
         Return a OneDSpectrum from a FITS HDU or HDU list.
+
+        Parameters
+        -----------
+        ext : int
+            The integer index to load when given an :class:`astropy.io.fits.HDUList`.
+            Default is 0 (the first HDU in the list.
+
         '''
 
         if isinstance(hdu, HDUList):
             hdul = hdu
-            hdu = hdul[0]
+            hdu = hdul[ext]
         else:
             hdul = HDUList([hdu])
 
@@ -652,9 +676,11 @@ class BaseOneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', category=FITSWarning)
             beam = cube_utils.try_load_beams(hdul)
-            if hasattr(beam, '__len__'):
+            try:
                 beams = beam
-            else:
+                _ = len(beams)
+            except TypeError:
+                # beam is scalar and has no len()
                 beams = None
 
         if beams is not None:
