@@ -19,6 +19,15 @@ from ..lower_dimensional_structures import (Projection, Slice, OneDSpectrum,
 from ..utils import SliceWarning, WCSCelestialError, BeamUnitsError
 from . import path
 
+# needed for regression in numpy
+import sys
+try:
+    from astropy.utils.compat import NUMPY_LT_1_22
+except ImportError:
+    # if astropy is an old version, we'll just skip the test
+    # (this is only used in one place)
+    NUMPY_LT_1_22 = False
+
 # set up for parametrization
 LDOs = (Projection, Slice, OneDSpectrum)
 LDOs_2d = (Projection, Slice,)
@@ -128,7 +137,9 @@ def test_isnan(LDO, data):
 def test_self_arith(LDO, data):
 
     image = data
-    p = LDO(image, copy=False)
+    p = LDO(image, copy=False, wcs=WCS(naxis=image.ndim))
+    assert hasattr(p, '_wcs')
+    assert p.wcs is not None
 
     p2 = p + p
 
@@ -150,14 +161,18 @@ def test_self_arith_with_beam(LDO, data):
     exp_beam = Beam(1.0 * u.arcsec)
 
     image = data
-    p = LDO(image, copy=False)
+    p = LDO(image, copy=False, wcs=WCS(naxis=image.ndim))
     p = p.with_beam(exp_beam)
+    assert hasattr(p, 'beam')
+    assert hasattr(p, '_wcs')
+    assert p.wcs is not None
 
     p2 = p + p
 
     assert hasattr(p2, '_wcs')
     assert p2.wcs == p.wcs
     assert np.all(p2.value==2)
+    assert hasattr(p2, 'beam')
     assert p2.beam == exp_beam
 
     p2 = p - p
@@ -165,6 +180,7 @@ def test_self_arith_with_beam(LDO, data):
     assert hasattr(p2, '_wcs')
     assert p2.wcs == p.wcs
     assert np.all(p2.value==0)
+    assert hasattr(p2, 'beam')
     assert p2.beam == exp_beam
 
 
@@ -697,6 +713,9 @@ def test_1d_slices(data_255_delta, use_dask):
     assert not isinstance(sp.max(), OneDSpectrum)
 
 
+# TODO: Unpin when Numpy bug is resolved.
+@pytest.mark.skipif(not NUMPY_LT_1_22 and sys.platform == 'win32',
+                    reason='https://github.com/numpy/numpy/issues/20699')
 @pytest.mark.parametrize('method',
                          ('min', 'max', 'std', 'mean', 'sum', 'cumsum',
                           'nansum', 'ptp', 'var'),
@@ -708,9 +727,12 @@ def test_1d_slice_reductions(method, data_255_delta, use_dask):
     sp = cube[:,0,0]
 
     if hasattr(cube, method):
-        assert getattr(sp, method)() == getattr(cube, method)(axis=0)[0,0]
+        spmethod = getattr(sp, method)
+        cubemethod = getattr(cube, method)
+        assert spmethod() == cubemethod(axis=0)[0,0]
     else:
-        getattr(sp, method)()
+        method = getattr(sp, method)
+        result = method()
 
     assert hasattr(sp, '_fill_value')
 
@@ -728,8 +750,11 @@ def test_1d_slice_round(data_255_delta, use_dask):
     assert hasattr(sp, '_fill_value')
     assert hasattr(sp.round(), '_fill_value')
 
-    assert 'OneDSpectrum' in sp.round().__repr__()
-    assert 'OneDSpectrum' in sp[1:-1].round().__repr__()
+    rnd = sp.round()
+    assert 'OneDSpectrum' in rnd.__repr__()
+
+    rndslc = sp[1:-1].round()
+    assert 'OneDSpectrum' in rndslc.__repr__()
 
 
 def test_LDO_arithmetic(data_vda, use_dask):
@@ -913,3 +938,16 @@ def test_spatial_world(view, data_adv, use_dask):
     for result, expected in zip(w2_flat, world):
         print(result.shape, expected.flatten().shape)
         assert_allclose(result, expected.flatten())
+
+@pytest.mark.parametrize(('LDO', 'data'),
+                         zip(LDOs, data_twelve))
+def test_unit_division(LDO, data):
+    # regression: 871
+
+    image = data
+    p = LDO(image, copy=False)
+
+    p._meta = None
+
+    # check that this does not raise an Exception
+    p.hdu
